package com.ccwangluo.ccim;

import android.Manifest;
import android.content.Context;
import android.os.Environment;
import android.text.TextUtils;
import androidx.fragment.app.FragmentActivity;
import com.ccwangluo.ccim.listener.ContactsCallBack;
import com.ccwangluo.ccim.listener.OnClientConnectListener;
import com.ccwangluo.ccim.listener.OnConnectListener;
import com.ccwangluo.ccim.listener.OnPushMsgListener;
import com.ccwangluo.ccim.listener.ResultCallback;
import com.ccwangluo.ccim.listener.ResultEventCallback;
import com.ccwangluo.ccim.manage.CallBackManager;
import com.ccwangluo.ccim.manage.ChannelManager;
import com.ccwangluo.ccim.manage.ContactManager;
import com.ccwangluo.ccim.manage.MessageManager;
import com.ccwangluo.ccim.manage.SocketManager;
import com.ccwangluo.ccim.modle.Contact;
import com.ccwangluo.ccim.modle.MessageContent;
import com.ccwangluo.ccim.modle.NetStatus;
import com.ccwangluo.ccim.modle.PushPrivateMsgRes;
import com.ccwangluo.ccim.modle.User;
import com.ccwangluo.ccim.util.ErrorCode;
import com.ccwangluo.ccim.util.MediaHelper;
import com.google.gson.Gson;
import com.google.protobuf.util.JsonFormat;
import com.permissionx.guolindev.PermissionX;
import java.io.File;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.logging.Logger;
import ccim.protocol.Chat;
import ccim.protocol.Message;
import ccim.protocol.MessageSpec;
import ccim.protocol.Push;

public class CCIMClient {

    private final Logger logger = Logger.getLogger(CCIMClient.class.getName());
    private static SocketManager socketManager;
    private static CallBackManager<Chat.ChatResponseMessage> callBackManager;
    private static ContactManager contactManager;
    private static ChannelManager channelManager;
    private static MessageManager messageManager;
    private static Long userId;
    private static boolean isConnected = false; //链接状态
    private Long lastSendTimemillis = 0L; //上次发送消息时间,用于时间间隔
    private int maxSendMsgInterval = 0;
    private int maxAudioDuration = 0;
    private int minAudioDuration = 500;
    private String downloadPrefix;
    private String cachePath;

    private ExecutorService executor = Executors.newSingleThreadExecutor();


    private CCIMClient() {
        callBackManager = new CallBackManager<>();
        socketManager = new SocketManager(callBackManager);
        contactManager = new ContactManager(socketManager, callBackManager);
        channelManager = new ChannelManager(socketManager, callBackManager);
        messageManager = new MessageManager(socketManager, callBackManager, executor);
    }

    private static CCIMClient mInstance;

    public static CCIMClient getInstance() {
        if (mInstance == null) {
            synchronized (CCIMClient.class) {
                if (mInstance == null) {
                    mInstance = new CCIMClient();
                }
            }
        }
        return mInstance;
    }

    public void initServer(String ip, int port) {
        socketManager.initServer(ip, port);
    }

    public void initSDK(Context context, String appKey, String userName, String extData, Long userId, Long ts, String sign, OnConnectListener resultCallback) {
        this.userId = userId;
        messageManager.init(context, userId);
        socketManager.initChat(context, appKey, userName, extData, userId, ts, sign, new OnClientConnectListener() {
            @Override
            public void onSocketConnection(Chat.ConnectResponse.ClientConfig clientConfig) {
                maxSendMsgInterval = clientConfig.getSendMsgInterval();
                maxAudioDuration = clientConfig.getMaxAudioDuration() * 1000;
                downloadPrefix = clientConfig.getDownloadPrefix();
                getContactList((contacts, contactGroups, dialog) -> {
                    if (!isConnected) {
                        resultCallback.onSocketConnection();
                        isConnected = true;
                    }
                }, true);
            }

            @Override
            public void onSocketReConnection() {
                isConnected = false;
            }

            @Override
            public void onSocketDisconnection() {
                isConnected = false;
                logger.info("disconnection");
                resultCallback.onSocketDisconnection();
            }
        });
    }


    public void registerMessageListener(OnPushMsgListener pushMsgListener) {
        callBackManager.addCallBack(0, chatResponseMessage -> {
            Gson gson = new Gson();
            if (chatResponseMessage.hasPushPrivateMsgRes()) {
                MessageSpec.ChatMessage pushPrivateMsgRes = chatResponseMessage.getPushPrivateMsgRes().getMessage();
                pushPrivateMessageAck(pushPrivateMsgRes.getSenderUserId(), pushPrivateMsgRes.getMsgId());
                try {
                    String print = JsonFormat.printer().print(chatResponseMessage.getPushPrivateMsgRes());
                    PushPrivateMsgRes pushPrivateMsgRes1 = gson.fromJson(print, PushPrivateMsgRes.class);

                    if (pushPrivateMsgRes1.getStrangerDialog() != null) {
                        contactManager.putStrangerDialog(pushPrivateMsgRes1.getStrangerDialog());
                    }
                    if (pushPrivateMsgRes1.getMessage() != null) {
                        MessageContent message = pushPrivateMsgRes1.getMessage();
                        Long senderUserId = message.getSenderUserId();
                        contactManager.putContactDialog(senderUserId, null);
                        pushPrivateMsgRes1.getMessage().setReciveUserId(userId);
                        pushMsgListener.pushPrivateMsg(pushPrivateMsgRes1.getMessage());
                        messageManager.pushNewMessage(pushPrivateMsgRes1.getMessage());
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            if (chatResponseMessage.hasPushChannelMsgRes()) {
                Push.PushChannelMessageResponse channelMsgRes = chatResponseMessage.getPushChannelMsgRes();
                pushChannelMessage(channelMsgRes.getChannelId(), channelMsgRes.getMessage().getMsgId());
                try {
                    String print = JsonFormat.printer().print(channelMsgRes.getMessage());
                    MessageContent messageContent = gson.fromJson(print, MessageContent.class);
                    messageContent.setChannelId(channelMsgRes.getChannelId());
                    pushMsgListener.pushChannelMsg(messageContent);
                    messageManager.pushNewMessage(messageContent);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            if (chatResponseMessage.hasPushNewContactRes()) {
                Push.PushNewContactResponse pushNewContactRes = chatResponseMessage.getPushNewContactRes();
                try {
                    String print = JsonFormat.printer().print(pushNewContactRes.getContact());
                    Contact contact = gson.fromJson(print, Contact.class);
                    contactManager.pushNewContact(contact);
                    pushMsgListener.pushNewContact(contact);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            if (chatResponseMessage.hasPushContactDeleteMeRes()) {
                Push.PushContactDeleteMeResponse pushNewContactRes = chatResponseMessage.getPushContactDeleteMeRes();
                long contactUserId = pushNewContactRes.getContactUserId();
                contactManager.pushDeleteContact(contactUserId);
                pushMsgListener.pushDeleteContact(contactUserId);
            }

            if (chatResponseMessage.hasPushContactNetStatusRes()) {
                Push.PushContactNetStatusResponse pushNewContactRes = chatResponseMessage.getPushContactNetStatusRes();
                long contactUserId = pushNewContactRes.getContactUserId();
                NetStatus netStatus = NetStatus.valueOf(pushNewContactRes.getNetStatus().name());
                contactManager.pushContactNetStatus(contactUserId, netStatus);
                pushMsgListener.pushContactNetStatus(contactUserId, netStatus);
            }

            if (chatResponseMessage.hasPushContactNewReqRes()) {
                Push.PushContactNewReqResponse pushContactNewReqRes = chatResponseMessage.getPushContactNewReqRes();
                String username = pushContactNewReqRes.getContactNewReq().getUsername();
                Long id = pushContactNewReqRes.getContactNewReq().getUserId();
                pushMsgListener.pushContactNewReq(new User(id, username));
            }

        });
    }


    /**
     * 获取联系人列表
     */
    public void getContactList(ContactsCallBack resultEventCallback, boolean isClearCache) {
        contactManager.getContactList(resultEventCallback, isClearCache);
    }


    /**
     * 搜索好友
     */
    public void searchUser(String userName, ResultEventCallback<List<User>> resultEventCallback) {
        contactManager.searchUser(userName, resultEventCallback);
    }

    /**
     * 添加好友
     */
    public void addContact(Long userId, ResultCallback callback) {
        contactManager.addContact(userId, callback);

    }

    /**
     * 联系人申请列表
     */
    public void getContactReqList(ResultEventCallback<List<User>> callback) {
        contactManager.getContactReqList(callback);

    }

    /**
     * 验证联系人申请
     */
    public void checkContactReq(Long reqUserId, boolean accept, ResultCallback callback) {
        contactManager.checkContactReq(reqUserId, accept, callback);
    }

    /**
     * 创建联系人群组
     */
    public void createContactGroup(String name, ResultEventCallback<Long> callback) {
        contactManager.createContactGroup(name, callback);
    }


    /**
     * 删除联系人群组
     */
    public void deleteContactGroup(Long groupId, ResultCallback callback) {
        contactManager.deleteContactGroup(groupId, callback);
    }


    /**
     * 移动联系人所在的群组
     */
    public void moveContactGroup(Long groupId, Long userId, ResultCallback callback) {
        contactManager.moveContactGroup(groupId, userId, callback);
    }

    /**
     * 删除联系人
     */
    public void deleteContact(Long userId, ResultCallback callback) {
        contactManager.deleteContact(userId, callback);
    }

    /**
     * 拉黑联系人
     */
    public void blockContact(Long userId, ResultCallback callback) {
        contactManager.blockContact(userId, callback);
    }

    /**
     * 举报违规用户
     */
    public void reportIllegalUser(String title, String content, Long userId, ResultCallback callback) {
        contactManager.reportIllegalUser(title, content, userId, callback);
    }


    /**
     * 创建频道
     */
    public void createChannel(String channelName, String tag, boolean isOpen, List<Long> userIds, ResultEventCallback<Long> callback) {
        channelManager.createChannel(channelName, tag, isOpen, userIds, callback);
    }

    /**
     * 加入频道
     */
    public void joinChannel(Long channelId, ResultCallback callback) {
        channelManager.joinChannel(channelId, callback);
    }

    /**
     * 移除频道成员
     */
    public void removeChannelMember(Long channelId, Long userId, ResultCallback callback) {
        channelManager.removeChannelMember(channelId, userId, callback);
    }

    /**
     * 频道拉黑
     */
    public void blockChannelUser(Long channelId, Long userId, ResultCallback callback) {
        channelManager.blockChannelUser(channelId, userId, callback);
    }

    /**
     * 退出频道
     */
    public void exitChannel(Long channelId, ResultCallback callback) {
        channelManager.exitChannel(channelId, callback);
    }

    /**
     * 同步离线消息
     */
    private void getOfflineMessage(Long lastPullMsgId, ResultEventCallback<List<Message.PullOfflineMessageResponse.OfflineMessage>> resultEventCallback) {
        messageManager.getOfflineMessage(lastPullMsgId, resultEventCallback);
    }

    /**
     * 校验时间间隔
     *
     * @return
     */
    private boolean checkTimeInterval() {
        if (lastSendTimemillis != 0) {
            long currentTimeMillis = System.currentTimeMillis();
            if (currentTimeMillis - lastSendTimemillis < maxSendMsgInterval) {
                return false;
            }
        }
        lastSendTimemillis = System.currentTimeMillis();
        return true;
    }

    /**
     * 发送消息
     */
    public void sendTextMessage(String content, Long receiveUserId, String receiveUserName, ResultEventCallback<MessageContent> callback) {
        if (TextUtils.isEmpty(content)) {
            callback.onFailed(ErrorCode.MSG_CONTENT_IS_EMPTY);
            return;
        }

        if (!checkTimeInterval()) {
            callback.onFailed(ErrorCode.SEND_MSG_INTERVAL);
            return;
        }
        messageManager.sendPrivateMessage(content, receiveUserId, MessageSpec.MessageType.MESSAGE_TYPE_TEXT, new ResultEventCallback<MessageContent>() {

            @Override
            public void onSuccess(MessageContent messageContent) {
                contactManager.putContactDialog(receiveUserId, receiveUserName);
                callback.onSuccess(messageContent);
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 发送频道消息
     */
    public void sendChannelMessage(String content, Long channelId, ResultEventCallback<MessageContent> callback) {
        if (TextUtils.isEmpty(content)) {
            callback.onFailed(ErrorCode.MSG_CONTENT_IS_EMPTY);
            return;
        }

        if (!checkTimeInterval()) {
            callback.onFailed(ErrorCode.SEND_MSG_INTERVAL);
            return;
        }
        messageManager.sendChannelMessage(content, channelId, MessageSpec.MessageType.MESSAGE_TYPE_TEXT, callback);
    }

    /**
     * 推送私人消息成功ack
     */
    public void pushPrivateMessageAck(Long contactUserId, Long msgId) {
        Chat.ChatRequestMessage.Builder builder = Chat.ChatRequestMessage.newBuilder();
        Push.PushPrivateMessageAckRequest pushPrivateMessageAckRequest = Push.PushPrivateMessageAckRequest.newBuilder()
                .setContactUserId(contactUserId)
                .setMsgId(msgId)
                .build();
        builder.setPushPrivateMsgAckReq(pushPrivateMessageAckRequest);
        socketManager.sendRequest(builder);
    }


    /**
     * 推送渠道消息成功ack
     */
    public void pushChannelMessage(Long channelId, Long msgId) {
        Chat.ChatRequestMessage.Builder builder = Chat.ChatRequestMessage.newBuilder();
        Push.PushChannelMessageAckRequest channelMessageAckRequest = Push.PushChannelMessageAckRequest.newBuilder()
                .setChannelId(channelId)
                .setMsgId(msgId)
                .build();
        builder.setPushChannelMsgAckReq(channelMessageAckRequest);
        socketManager.sendRequest(builder);
    }

    /**
     * 开始录音
     */

    public void startRecordAudioMessage(FragmentActivity activity, ResultCallback resultCallback) {
        PermissionX.init(activity)
                .permissions(Manifest.permission.RECORD_AUDIO)
                .onExplainRequestReason((scope, deniedList, beforeRequest) -> scope.showRequestReasonDialog(deniedList, "即将申请的权限是程序必须依赖的权限", "我已明白"))
                .onForwardToSettings((scope, deniedList) -> scope.showForwardToSettingsDialog(deniedList, "您需要去应用程序设置当中手动开启权限", "我已明白"))
                .request((allGranted, grantedList, deniedList) -> {
                    if (allGranted) {
                        MediaHelper.getInstance().prepare(cachePath,maxAudioDuration, resultCallback);
                    } else {
                        resultCallback.onFailed(ErrorCode.START_VOICE_FAILED);
                    }
                });
    }

    /**
     * 停止录音并上传 私聊
     */
    public void stopAndSendAudioMessage(Long receiveUserId, String receiveUserName, ResultEventCallback<MessageContent> callback) {
        if (checkAudio(callback)) {
            return;
        }
        messageManager.uploadFileMessage(cachePath, new ResultEventCallback<String>() {
            @Override
            public void onSuccess(String fileName) {
                messageManager.sendPrivateMessage(fileName, receiveUserId, MessageSpec.MessageType.MESSAGE_TYPE_VOICE, new ResultEventCallback<MessageContent>() {
                    @Override
                    public void onSuccess(MessageContent messageContent) {
                        contactManager.putContactDialog(receiveUserId, receiveUserName);
                        callback.onSuccess(messageContent);
                    }

                    @Override
                    public void onFailed(int errorCode) {
                        callback.onFailed(errorCode);
                    }
                });
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 停止录音并上传 频道
     */
    public void stopAndSendChannelAudioMessage(Long channelID, ResultEventCallback<MessageContent> callback) {
        if (checkAudio(callback)) {
            return;
        }
        messageManager.uploadFileMessage(cachePath, new ResultEventCallback<String>() {
            @Override
            public void onSuccess(String fileName) {
                messageManager.sendChannelMessage(fileName, channelID, MessageSpec.MessageType.MESSAGE_TYPE_VOICE, callback);
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    private boolean checkAudio(ResultEventCallback<MessageContent> callback) {
        Long release = MediaHelper.getInstance().release();
        if (release < minAudioDuration) {
            callback.onFailed(ErrorCode.VOICE_TOO_SHORT);
            MediaHelper.getInstance().cancel();
            return true;
        }
        if (release > maxAudioDuration) {
            callback.onFailed(ErrorCode.VOICE_TOO_LONG);
            MediaHelper.getInstance().cancel();
            return true;
        }
        return false;
    }

    /**
     * 取消录音
     */
    public void cancleAudioMessage() {
        MediaHelper.getInstance().cancel();
    }

    /**
     * 播放音乐
     */
    public void startPlayAudio(Long msgId, ResultCallback resultCallback) {
        messageManager.playAudio(msgId, downloadPrefix, cachePath, resultCallback);
    }

    public void logout() {
        isConnected = false;
        socketManager.disConnect();
    }
}
